package Report;

use strict;
use warnings;

use Exporter;
use Html qw#head foot escape navbar get_filename get_title#;
use Data::Core qw#receive get_values#;
use Data::Setup qw#@SELECTS#;
use Data::Convert qw#convert#;
use Config qw#get_config#;

@Report::EXPORT_OK = qw#generate#;
@Report::ISA = ('Exporter');

my $VERSION = '$Id: Classic.pm,v 1.2 2003/03/06 14:10:40 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

our $OUTPUTDIR = get_config ('directory');
if ($OUTPUTDIR)
{
	$OUTPUTDIR =~ s#/$##;
	if (!-d $OUTPUTDIR)
	{
		print STDERR $/, __FILE__, qq#: Directory "$OUTPUTDIR" does not exist.\n#;
		exit (1);
	}
	$OUTPUTDIR .= '/';
}
elsif (-d 'reports')
{
	$OUTPUTDIR = 'reports/';
}
else
{
	print STDERR $/, __FILE__, ": Unknown output directory. Using current ",
		"directory instead!",
		$/, __FILE__, ": To abort press CTRL+c within the next 10 secons";
	
	sleep 10;

	$OUTPUTDIR = './';
}

return (1);

sub generate
{
	for (@SELECTS)
	{
		my $sel = $_;
		my @keys = @{$sel->[1]};

		generate_sub_index ($sel);

		for (@keys)
		{
			my $key = $_;
			generate_sub_page ($sel, $key);
		}
	}

	generate_index_page ();

	return (1);
}

sub generate_sub_page
{
	my $sel = shift;
	my $key = shift;

	my @vals = get_values ($sel, $key);

	my $filename = get_filename ($sel);
	$filename =~ s/\.html$/__$key.html/;
	my $title = get_title ($sel);

	open (FH, '> ' . $OUTPUTDIR . $filename) or die ('open: ' . $!);
	
	print FH head ($title, $title);
	print FH '<h2>', ucfirst ($key), "</h2>\n";
	print FH qq#<a id="top_of_page"></a>\n#;
	print FH navbar ($sel);
	print FH own_navbar ($sel, $key);
	
	for (sort (@vals))
	{
		my $val = $_;
		print FH generate_table ($sel, $key, $val);
	}

	print FH foot ();
	close (FH);
}

sub generate_sub_index
{
	my $sel = shift;

	my $filename = get_filename ($sel);
	my $title = get_title ($sel);

	open (FH, '> ' . $OUTPUTDIR . $filename) or die ('open: ' . $!);

	print FH head ($title, $title);
	print FH navbar ($sel);
	print FH own_navbar ($sel);

	# TODO
	
	print FH foot ();
	close (FH);
}

sub generate_index_page
{
	open (FH, '> ' . $OUTPUTDIR . 'index.html') or die ('open: ' . $!);

	print FH head ("yaala $::VERSION", "yaala $::VERSION - Index");
	print FH navbar ();

	if (scalar (keys (%$::EXTRA)))
	{
		print FH "\n<hr>\n<table>\n";
		for (keys (%$::EXTRA))
		{
			my $key = $_;
			my $val = $::EXTRA->{$key};

			print FH qq#  <tr>\n    <th class="top">$key</th>\n    <td>$val</td>\n  </tr>\n#;
		}
		print FH "</table>\n";
	}
	else
	{
		print FH "\n<!-- no \%::EXTRA -->\n";
	}
	
	print FH foot ();
	close (FH);
}

sub generate_table
{
	my $sel = shift;
	my $key = shift;
	my $val = shift;
	
	my $agg = $sel->[0];

	my @keys = grep { $_ ne $key } (@{$sel->[1]});
	@keys = sort (@keys);

	my $link_val = $val;
	$link_val =~ s/\W//g;
	
	my $text = qq#\n<hr>\n<a id="$link_val"></a>\n<table>\n  <tr>\n#;
	$text .= '    <th colspan="3">' . ucfirst ($val) . "</th>\n  </tr>\n";
	$text .= qq#  <tr>\n    <th class="subhdr">Field</th>\n#
	.	qq#    <th class="subhdr">Value</th>\n#
	.	qq#    <th class="subhdr"># . ucfirst ($agg) . "</th>\n  </tr>\n";

	for (@keys)
	{
		my $second_key = $_;
		my @second_vals = get_values ($sel, $second_key);

		my $first_line = 1;
		
		my $num_vals = scalar (@second_vals);

		$text .= qq#  <tr>\n    <th class="subhdr"#
		. ($num_vals > 1 ? qq# rowspan="$num_vals"# : '')
		. '>' . ucfirst ($second_key) . "</th>\n";
		
		for (sort (@second_vals))
		{
			my $this_val = $_;

			if (!$first_line) { $text .= "  <tr>\n"; }
			$first_line = 0;

			my $sum = receive ($sel, {$key => $val, $second_key => $this_val});
			my $print_sum = convert ($agg, $sum);

			$text .= qq#    <td>$this_val</td>\n#
			.	"    <td>"
			.	($print_sum ? $print_sum : '&nbsp;' )
			.	qq#</td>\n#
			.	qq#  </tr>\n#;
		}
	}

	$text .= "</table>\n"
	.	qq(<p>[&nbsp;<a href="#top_of_page">top</a>&nbsp;]</p>\n);

	return ($text);
}

sub own_navbar
{
	my $sel = shift;
	my $key = shift;

	if (!defined ($key)) { $key = ''; }

	my $base_filename = get_filename ($sel);
	my $text = "<p>\n";

	for (@{$sel->[1]})
	{
		my $this_key = $_;
		my $this_filename = $base_filename;
		$this_filename =~ s/\.html$/__$this_key.html/;

		if ($this_key eq $key)
		{
			$text .= '  [&nbsp;' . ucfirst ($key) . "&nbsp;]\n";
		}
		else
		{
			$text .= qq#  [&nbsp;<a href="$this_filename"># . ucfirst ($this_key) . "</a>&nbsp;]\n";
		}
	}

	$text .= "</p>\n";

	if ($key)
	{
		my @vals = get_values ($sel, $key);

		$text .= "<p>\n";
		for (sort (@vals))
		{
			my $link_val = $_;
			my $print_val = convert ($key, $_);
			$link_val =~ s/\W//g;
			$print_val =~ s/\s/&nbsp;/g;
		
			$text .= qq(  [&nbsp;<a href="#$link_val">$print_val</a>&nbsp;]\n);
		}
		$text .= "</p>\n";
	}
	
	return ($text);
}
